DJU_EMAIL_BLOCK_MARKERS = {
    'subject': ('<!--__DJU_EMAIL_SUBJECT_BEGIN__-->', '<!--__DJU_EMAIL_SUBJECT_END__-->'),
    'plain_body': ('<!--__DJU_EMAIL_PLAIN_BODY_BEGIN__-->', '<!--__DJU_EMAIL_PLAIN_BODY_END__-->'),
    'html_body': ('<!--__DJU_EMAIL_HTML_BODY_BEGIN__-->', '<!--__DJU_EMAIL_HTML_BODY_END__-->'),
}
