{% load dju_email %}


{% dju_email_subject %}Test email from {{ SITE.domain }}{% end_dju_email_subject %}


{% dju_email_plain_body %}
    Hi!

    This is the test email. It helps to test sending email from Your project.

    Good luck!

    --------
    Please don't reply to this email.
{% end_dju_email_plain_body %}


{% dju_email_html_body %}
    <!DOCTYPE html>
    <html>
    <head></head>
    <body style="font-size: 16px; line-height: 1.5;">
        <p style="font-size: 18px;">
            Hi!
        </p>
        <p>
            This is the test email. It helps to test sending email from Your project.
        </p>
        <p style="font-size: 18px;">
            Good luck!
        </p>
        <p style="font-size: 13px; color: #555555;">
            Please don't reply to this email.
        </p>
    </body>
    </html>
{% end_dju_email_html_body %}