<!DOCTYPE html>
<html>
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
    <title>{{ title|default_if_none:'' }}</title>
    {{ head|default_if_none:'' }}
</head>
<body>
    {{ content }}
</body>
</html>